@@ -575,12 +575,16 @@ enum {
         TASK_COLUMN_COMPLETED,
         TASK_COLUMN_COMPLETED_TIME,
         TASK_COLUMN_OVERDUE_ATTR,
+        TASK_COLUMN_URL,
         N_TASK_COLUMNS
 };
 
 static char *
 format_time (ClockFormat format,
              time_t      t,
+             guint       year,
+             guint       month,
+             guint       day,
              gboolean    use_utc)
 {
         struct tm *tm;
@@ -597,13 +601,22 @@ format_time (ClockFormat format,
         if (!tm)
                 return NULL;
 
-        if (format != CLOCK_FORMAT_12 && format != CLOCK_FORMAT_24)
-                format = clock_locale_format ();
+        if (year  == (tm->tm_year + 1900) &&
+            month == tm->tm_mon &&
+            day   == tm->tm_mday) {
+                if (format != CLOCK_FORMAT_12 && format != CLOCK_FORMAT_24)
+                        format = clock_locale_format ();
 
-        if (format == CLOCK_FORMAT_12)
-                time_format = g_locale_from_utf8 (_("%l:%M %p"), -1, NULL, NULL, NULL);
-        else
-                time_format = g_locale_from_utf8 (_("%H:%M"), -1, NULL, NULL, NULL);
+                if (format == CLOCK_FORMAT_12)
+                        time_format = g_locale_from_utf8 (_("%l:%M %p"), -1, NULL, NULL, NULL);
+                else
+                        time_format = g_locale_from_utf8 (_("%H:%M"), -1, NULL, NULL, NULL);
+        }
+        else {
+                /* Translators: If the event did not start on the current day
+                 we will display the start date in the most abbreviated way possible. */
+                time_format = g_locale_from_utf8 (_("%b %d"), -1, NULL, NULL, NULL);
+        }
 
         strftime (result, sizeof (result), time_format, tm);
 
@@ -641,6 +654,7 @@ handle_tasks_changed (ClockData *cd)
                                     TASK_COLUMN_PERCENT_COMPLETE_TEXT, percent_complete_text,
                                     TASK_COLUMN_COMPLETED,             task->percent_complete == 100,
                                     TASK_COLUMN_COMPLETED_TIME,        task->completed_time,
+                                    TASK_COLUMN_URL,                   task->url,
                                     -1);
 
                 g_free (percent_complete_text);
@@ -813,6 +827,26 @@ create_hig_frame (const char  *title)
         return vbox;
 }
 
+static gboolean
+task_activated_cb (GtkTreeView       *view,
+                   GtkTreePath       *path,
+                   GtkTreeViewColumn *column,
+                   ClockData         *cd)
+{
+        GError     *err = NULL;
+        GtkTreeIter iter;
+        char       *uri;
+
+        gtk_tree_model_get_iter (GTK_TREE_MODEL (cd->tasks_model), &iter, path);
+        gtk_tree_model_get (GTK_TREE_MODEL (cd->tasks_model), &iter,
+                            TASK_COLUMN_URL, &uri, -1);
+
+        if (uri)
+                gnome_url_show (uri, &err);
+
+        return TRUE;
+}
+
 static GtkWidget *
 create_task_list (ClockData  *cd,
                   GtkWidget **tree_view,
@@ -846,7 +880,8 @@ create_task_list (ClockData  *cd,
                         G_TYPE_STRING,         /* percent complete text   */
                         G_TYPE_BOOLEAN,        /* completed               */
                         G_TYPE_LONG,           /* completed time          */
-                        PANGO_TYPE_ATTR_LIST   /* summary text attributes */
+                        PANGO_TYPE_ATTR_LIST,  /* summary text attributes */
+                        G_TYPE_STRING          /* url                     */
                 };
 
                 cd->tasks_model = gtk_list_store_newv (N_TASK_COLUMNS, column_types);
@@ -872,7 +907,10 @@ create_task_list (ClockData  *cd,
 
         *tree_view = view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (cd->tasks_filter));
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
-  
+
+        g_signal_connect (view, "row-activated",
+                          G_CALLBACK (task_activated_cb), cd);
+
         /* Completed toggle */
         column = gtk_tree_view_column_new ();
         cell = gtk_cell_renderer_toggle_new ();
@@ -930,6 +968,7 @@ static void
 handle_appointments_changed (ClockData *cd)
 {
         GSList *events, *l;
+        guint   year, month, day;
 
         if (cd->calendar) {
                 gtk_calendar_freeze (GTK_CALENDAR (cd->calendar));
@@ -944,6 +983,7 @@ handle_appointments_changed (ClockData *cd)
         gtk_list_store_clear (cd->appointments_model);
 
         events = calendar_client_get_events (cd->client, CALENDAR_EVENT_APPOINTMENT);
+        calendar_client_get_date (cd->client, &year, &month, &day);
         for (l = events; l; l = l->next) {
                 CalendarAppointment *appointment = l->data;
                 GtkTreeIter          iter;
@@ -951,12 +991,14 @@ handle_appointments_changed (ClockData *cd)
 
                 g_assert (CALENDAR_EVENT (appointment)->type == CALENDAR_EVENT_APPOINTMENT);
 
-                if (!appointment->is_all_day)
+                if (appointment->is_all_day)
+                        start_text = g_strdup (_("All Day"));
+                else
                         start_text = format_time (cd->format,
                                                   appointment->start_time,
+                                                  year, month, day,
                                                   cd->gmt_time);
-                else
-                        start_text = g_strdup (_("All Day"));
+
 
                 gtk_list_store_append (cd->appointments_model, &iter);
                 gtk_list_store_set (cd->appointments_model, &iter,
@@ -1198,7 +1240,7 @@ create_calendar (ClockData *cd,
         gtk_window_set_screen (window, screen);
 
         frame = gtk_frame_new (NULL);
-        gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+        gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
         gtk_container_add (GTK_CONTAINER (window), frame);
         gtk_widget_show (frame);
 
