@@ -83,8 +83,8 @@ GdkPixbuf *	missing_pixbuf		(int size);
 
 char *panel_make_full_path   (const char *dir,
 			      const char *filename);
-char *panel_make_unique_path (const char *dir,
-			      const char *suffix);
+char *panel_make_unique_uri (const char *dir,
+			     const char *suffix);
 
 G_END_DECLS
 
